<?php
function enomssl_config($params=[])
{
    $isExist = GetEmailTemplateInfo("SSL Certificate Configuration Required");
    if (empty($isExist))
    {
        $subject = $name = "SSL Certificate Configuration Required";
        $message = "<p>Dear {\$client_name},</p><p>Thank you for your order for an SSL Certificate. Before you can use your certificate, it requires configuration which can be done at the URL below.</p><p>{\$ssl_configuration_link}</p><p>Instructions are provided throughout the process but if you experience any problems or have any questions, please open a ticket for assistance.</p><p>{\$signature}</p>";
        AddEditEmailTemplate(["name"=>$name,"type"=>"product","is_plain_text"=>false,"is_disabled"=>false,"is_custom"=>false,"subject"=>$subject,"message"=>$message]);
    }
    $enomusername = $enompassword = $testmode = "";
    $product = $params["product"];
    $configurations = $product["configurations"];
    $enomusername = (!empty(@$configurations["configoption1"])) ? $configurations["configoption1"]:"";
    $enompassword = (!empty(@$configurations["configoption2"])) ? $configurations["configoption2"]:"";
    $testmode = (@$configurations["configoption5"] == "yes") ? true:false;


    if ($enomusername && $enompassword)
    {
        $postfields = array();
        $postfields["uid"] = $enomusername;
        $postfields["pw"] = $enompassword;
        $postfields["command"] = "GetCerts";
        $postfields["ResponseType"] = "XML";
        $result = enomssl_call($postfields, $testmode);
        $certtypelist = "";
        foreach ($result["INTERFACE-RESPONSE"]["GETCERTS"]["CERTS"] as $cert => $details) {
            $certcode = $details["PRODCODE"];
            if ($certcode) {
                $certcode = str_replace("-", " ", $certcode);
                $certcode = titleCase($certcode);
                $certtypelist .= $certcode . ",";
            }
        }
        $certtypelist = substr($certtypelist, 0, -1);
        if (!$certtypelist) {
            $certtypelist = "certificate-rapidssl-rapidssl,certificate-geotrust-quickssl,certificate-geotrust-quickssl-premium,certificate-geotrust-truebizid,certificate-geotrust-truebizid-ev,certificate-geotrust-truebizid-wildcard,certificate-verisign-secure-site,certificate-verisign-secure-site-pro,certificate-verisign-secure-site-ev,certificate-verisign-secure-site-pro-ev,certificate-comodo-essential,certificate-comodo-premium-wildcard,certificate-comodo-essential-wildcard,certificate-comodo-ev,certificate-comodo-ev-sgc,certificate-comodo-ucc-dv-1yr-additional-domain,certificate-comodo-ucc-dv-2yr-additional-domain,certificate-comodo-ucc-dv-3yr-additional-domain,certificate-comodo-ucc-ov,certificate-comodo-ucc-ov-1yr-additional-domain,certificate-comodo-ucc-ov-2yr-additional-domain,certificate-comodo-ucc-ov-3yr-additional-domain";
        }
    } else {
        $certtypelist = "Please Enter your Username and Password";
    }

    return array(
        "name" => "eNom SSL Certificates",
        "description" => "Provision SSL Certificates with an Enom account.",
        "version" => "1.0",
        "author" => "WHMSOLS",
        "author_url"=>"",
        "RequiresServer"=>false,//true or false
        "AutoGenerateUsernameAndPassword"=>false,//true or false
        "DefaultNonSSLPort"=>'',
        "DefaultSSLPort"=>'',
        "WebmailSSLPort"=>'',
        "WebmailNonSSLPort"=>'',
        "ServiceSingleSignOnLabel"=>'',
        "AdminSingleSignOnLabel"=>'',
        //"PackagesLoader"=>'PackagesLoader',
        "configs" =>[
            "configoption1" =>["Name"=>"Username","Type" => "text", "Size" => "25"],
            "configoption2" =>["Name"=>"Password","Type" => "password", "Size" => "25"],
            "configoption3" =>["Name"=>"Certificate Type","Type" => "dropdown", "Options" => $certtypelist],
            "configoption4" =>["Name"=>"Years","Type" => "dropdown", "Options" => "1,2,3,4,5,6,7,8,9,10"],
            "configoption5" =>["Name"=>"Test Mode","Type" => "yesno"]
        ]);
}
function enomssl_CreateAccount($params)
{
    $product = $params["product"];
    $client = $params["client"];
    $configurations = $product["configurations"];
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    $postfields["EmptyCart"] = "On";
    $postfields["command"] = "DeleteFromCart";
    $postfields["ResponseType"] = "XML";
    $testmode = (@$configurations["configoption5"] == "yes") ? true:false;
    $clearResult = enomssl_call($postfields, $testmode);
    if (!is_array($clearResult) && substr($clearResult, 0, 4) == "CURL")
    {
        return ["status"=>"error","message"=>$clearResult];
    }
    $error = $clearResult["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
    if ($error && strpos($error, "There are no items in the cart to be deleted") === false) {
        return ["status"=>"error","message"=>$error];
    }
    $certtype = $params["config_options"]["Certificate Type"] ? $params["config_options"]["Certificate Type"] : $configurations["configoption3"];
    $certyears = $params["config_options"]["Years"] ? $params["config_options"]["Years"] : $configurations["configoption4"];
    $certtype = str_replace(" ", "-", strtolower($certtype));
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    $postfields["ProductType"] = $certtype;
    $postfields["Quantity"] = $certyears;
    $postfields["ClearItems"] = "yes";
    $postfields["command"] = "AddToCart";
    $postfields["ResponseType"] = "XML";
    $result = enomssl_call($postfields, $testmode);
    if (!is_array($result) && substr($result, 0, 4) == "CURL")
    {
        return ["status"=>"error","message"=>$result];
    }
    $error = $result["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
    if ($error) {
        return ["status"=>"error","message"=>$error];
    }
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    $postfields["command"] = "InsertNewOrder";
    $postfields["ResponseType"] = "XML";
    $result = enomssl_call($postfields, $testmode);
    $error = $result["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
    if ($error) {
        return ["status"=>"error","message"=>$error];
    }
    $orderid = $result["INTERFACE-RESPONSE"]["ORDERID"];
    AddUpdateSslInfo(["userid"=>$client["userid"],"remoteid"=>$orderid,"status"=>"Awaiting Configuration","serviceid"=>$params["serviceid"],"module"=>"enomssl","certtype"=>$certtype],["userid"=>$client["userid"],"serviceid"=>$params["serviceid"],"module"=>"enomssl"]);
    sendMessage("SSL Certificate Configuration Required", $params["serviceid"], array("ssl_configuration_link" => $sslOrder->getConfigurationUrl()));
    return ["status"=>"success","message"=>""];
}
function enomssl_TerminateAccount($params)
{
    $product = $params["product"];
    $client = $params["client"];
    $configurations = $product["configurations"];    
    $SslInfo = GetSslInfo(["serviceid"=>$params["serviceid"],"module"=>"enomssl","status"=>"Awaiting Configuration"]);
    if (!$SslInfo)
    {
        return ["status"=>"error","message"=>"SSL Either not Provisioned or Not Awaiting Configuration so unable to cancel"];
    }
    AddUpdateSslInfo(["status"=>"Cancelled"],["userid"=>$client["userid"],"serviceid"=>$params["serviceid"],"module"=>"enomssl"]);
    return ["status"=>"success","message"=>"SSL Terminated Successfully"];
}
function enomssl_AdminCustomButtonArray()
{
    $buttonarray = array("Resend Configuration Email" => "resend");
    return $buttonarray;
}
function enomssl_resend($params)
{
    $product = $params["product"];
    $client = $params["client"];
    $configurations = $product["configurations"];    

    $SslInfo = GetSslInfo(["serviceid"=>$params["serviceid"],"module"=>"enomssl","status"=>"Awaiting Configuration"]);
    if (!$SslInfo) return ["status"=>"error","message"=>"No SSL Order exists for this product"];
    sendMessage("SSL Certificate Configuration Required", $params["serviceid"], array("ssl_configuration_link" => $sslOrder->getConfigurationUrl()));
    return ["status"=>"success","message"=>"Email Sent Successfully"];
}
function enomssl_ClientArea($params)
{
    global $_LANG;
    $product = $params["product"];
    $client = $params["client"];
    $configurations = $product["configurations"];    
    $sslOrder = GetSslInfo(["serviceid"=>$params["serviceid"],"module"=>"enomssl"]);
    $output = "";
    if ($sslOrder) {
        $id = $sslOrder->id;
        $provisiondate = $product["reg_date"];
        $status = $sslOrder->status;

        $status .= " - <a href=\"configuressl.php?cert=" . md5($id) . "\">" . $_LANG["sslconfigurenow"] . "</a>";
        $output = "<div align=\"left\">\n<table width=\"100%\">\n<tr><td width=\"150\" class=\"fieldlabel\">" . $_LANG["sslprovisioningdate"] . ":</td><td>" . $provisiondate . "</td></tr>\n<tr><td class=\"fieldlabel\">" . $_LANG["sslstatus"] . ":</td><td>" . $status . "</td></tr>\n</table>\n</div>";
    }
    return $output;
}
function enomssl_AdminServicesTabFields($params)
{
    $product = $params["product"];
    $client = $params["client"];
    $configurations = $product["configurations"];    
    $sslOrder = GetSslInfo(["serviceid"=>$params["serviceid"],"module"=>"enomssl"]);
    $remoteid = "-";
    $status = "Not Yet Provisioned";
    if ($sslOrder)
    {
        $remoteid = $sslOrder->remoteId;
        $status = $sslOrder->status;
    }
    $fieldsarray = array("Enom Order ID" => $remoteid, "SSL Configuration Status" => $status);
    return $fieldsarray;
}
function enomssl_SSLStepOne($params)
{
    $product = $params["product"];
    $client = $params["client"];
    $configurations = $product["configurations"];    
    $testmode = (@$configurations["configoption5"] == "yes") ? true:false;
    $orderid = $params["remoteid"];
    $values = array();
    if (!$_SESSION["enomsslcert"][$orderid]["id"])
    {
        $postfields = array();
        $postfields["uid"] = $configurations["configoption1"];
        $postfields["pw"] = $configurations["configoption2"];
        $postfields["command"] = "CertGetCerts";
        $postfields["ResponseType"] = "XML";
        $result = enomssl_call($postfields, $testmode);
        $values["error"] = $result["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
        if ($values["error"]) return ["status"=>"error","message"=>$values["error"]];

        $cert_allowconfig = false;
        foreach ($result["INTERFACE-RESPONSE"]["CERTGETCERTS"]["CERTS"] as $certificate) {
            $temp_cert_id = $certificate["CERTID"];
            $temp_cert_name = $certificate["PRODDESC"];
            $temp_cert_status = $certificate["CERTSTATUS"];
            $temp_cert_orderid = $certificate["ORDERID"];
            $temp_cert_orderdate = $certificate["ORDERDATE"];
            $temp_cert_validityperiod = $certificate["VALIDITYPERIOD"];
            if ($temp_cert_orderid == $orderid) {
                $cert_id = $temp_cert_id;
                $cert_name = $temp_cert_name;
                $cert_orderid = $temp_cert_orderid;
                $cert_orderdate = $temp_cert_orderdate;
                $cert_validityperiod = $temp_cert_validityperiod;
                if ($temp_cert_status == "Awaiting Configuration" || $temp_cert_status == "Rejected by Customer") {
                    $cert_allowconfig = true;
                }
            }
        }
        $istatus = "";
        if (!$cert_allowconfig)
        {
            $istatus = "Completed";
        } else {
            $istatus = "Awaiting Configuration";
        }
        AddUpdateSslInfo(["status"=>$istatus],["userid"=>$client["userid"],"serviceid"=>$params["serviceid"],"module"=>"enomssl"]);
        $_SESSION["enomsslcert"][$orderid]["id"] = $cert_id;
    }
    else
    {
        $cert_id = $_SESSION["enomsslcert"][$orderid]["id"];
    }
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    $postfields["CertID"] = $cert_id;
    $postfields["command"] = "CertGetCertDetail";
    $postfields["ResponseType"] = "XML";
    $result = enomssl_call($postfields, $testmode);
    $values["error"] = $result["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
    if ($values["error"]) return ["status"=>"error","message"=>$values["error"]];

    $values["displaydata"]["Domain"] = $result["INTERFACE-RESPONSE"]["CERTGETCERTDETAIL"]["DOMAINNAME"];
    $values["displaydata"]["Validity Period"] = $result["INTERFACE-RESPONSE"]["CERTGETCERTDETAIL"]["VALIDITYPERIOD"] . " Months";
    $values["displaydata"]["Expiration Date"] = $result["INTERFACE-RESPONSE"]["CERTGETCERTDETAIL"]["EXPIRATIONDATE"];
    $certtype = $params["config_options"]["Certificate Type"] ? $params["config_options"]["Certificate Type"] : $configurations["configoption3"];
    $certtype = str_replace(" ", "-", strtolower($certtype));
    if (stristr($certtype, "-ucc-")) {
        $values["additionalfields"]["Additional Certificate Configuration"] = array(
            "ucc_domains" => array("FriendlyName" => "Domain Name List", "Type" => "textarea", "Rows" => "4", "Description" => "A list of Domains for this Certificate. One per line. Include the CSR Domain.", "Required" => true),
            "ucc_emails" => array("FriendlyName" => "Approval Emails List", "Type" => "textarea", "Rows" => "4", "Description" => "A list of Approval Emails for this Certificate. One per line for each domain.", "Required" => true)
        );
    }
    return $values;
}
function enomssl_SSLStepTwo($params)
{
    $product = $params["product"];
    $client = $params["client"];
    $configurations = $product["configurations"];    
    $testmode = (@$configurations["configoption5"] == "yes") ? true:false;

    $orderid = $params["remoteid"];
    $cert_id = $_SESSION["enomsslcert"][$orderid]["id"];
    $webservertype = $params["servertype"];
    $csr = $params["csr"];
    $firstname = $params["firstname"];
    $lastname = $params["lastname"];
    $organisationname = $params["orgname"];
    $jobtitle = $params["jobtitle"];
    $emailaddress = $params["email"];
    $address1 = $params["address1"];
    $address2 = $params["address2"];
    $city = $params["city"];
    $state = $params["state"];
    $postcode = $params["postcode"];
    $country = $params["country"];
    $phonenumber = $params["phonenumber"];
    $faxnumber = $params["faxnumber"];
    $certificateType = $params["config_options"]["Certificate Type"] ? $params["config_options"]["Certificate Type"] : $configurations["configoption3"];
    $certificateType = str_replace(" ", "-", strtolower($certificateType));
    $values = array();
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    $postfields["CertID"] = $cert_id;
    $postfields["WebServerType"] = $webservertype;
    $postfields["CSR"] = $csr;
    $contacttypes = array("Admin", "Tech", "Billing");
    foreach ($contacttypes as $contacttype) {
        $postfields[$contacttype . "FName"] = $firstname;
        $postfields[$contacttype . "LName"] = $lastname;
        $postfields[$contacttype . "OrgName"] = $organisationname;
        $postfields[$contacttype . "JobTitle"] = $jobtitle;
        $postfields[$contacttype . "Address1"] = $address1;
        $postfields[$contacttype . "Address2"] = $address2;
        $postfields[$contacttype . "City"] = $city;
        if ($country == "US") {
            $postfields[$contacttype . "State"] = $state;
        } else {
            $postfields[$contacttype . "Province"] = $state;
        }
        $postfields[$contacttype . "PostalCode"] = $postcode;
        $postfields[$contacttype . "Country"] = $country;
        $postfields[$contacttype . "Phone"] = $phonenumber;
        $postfields[$contacttype . "Fax"] = $faxnumber;
        $postfields[$contacttype . "EmailAddress"] = $emailaddress;
    }
    $showApprovalEmails = true;
    if (stristr($certificateType, "-ucc-")) {
        $count = 0;
        $uccDomainEmails = explode("\r\n", $params["fields"]["ucc_emails"]);
        foreach (explode("\r\n", $params["fields"]["ucc_domains"]) as $key => $domain) {
            $count++;
            $postfields["UCCDomainList" . $count] = $domain;
            $postfields["UCCEmailList" . $count] = $uccDomainEmails[$key];
        }
        $postfields["DomainListNumber"] = $count;
        $showApprovalEmails = false;
    }
    $postfields["command"] = "CertConfigureCert";
    $postfields["ResponseType"] = "XML";
    $result = enomssl_call($postfields, $testmode);
    $values["error"] = $result["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
    if ($values["error"]) return ["status"=>"error","message"=>$values["error"]];
    $approveremailsarray = array();
    if ($showApprovalEmails) {
        foreach ($result["INTERFACE-RESPONSE"]["CERTCONFIGURECERT"] as $k => $v) {
            if (substr($k, 0, 8) == "APPROVER") {
                $approver = trim($v["APPROVEREMAIL"]);
                if ($approver) {
                    $approveremailsarray[] = $approver;
                }
            }
        }
    }
    $values["approveremails"] = $approveremailsarray;
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    $postfields["CertID"] = $cert_id;
    $postfields["command"] = "CertGetCertDetail";
    $postfields["ResponseType"] = "XML";
    $result = enomssl_call($postfields, $testmode);
    $values["error"] = $result["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
    if ($values["error"]) return ["status"=>"error","message"=>$values["error"]];

    $values["displaydata"]["Domain"] = $result["INTERFACE-RESPONSE"]["CERTGETCERTDETAIL"]["DOMAINNAME"];
    $values["displaydata"]["Validity Period"] = $result["INTERFACE-RESPONSE"]["CERTGETCERTDETAIL"]["VALIDITYPERIOD"] . " Months";
    $values["displaydata"]["Expiration Date"] = $result["INTERFACE-RESPONSE"]["CERTGETCERTDETAIL"]["EXPIRATIONDATE"];
    $params["model"]->serviceProperties->save(array("domain" => $values["displaydata"]["Domain"]));
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    $postfields["CertID"] = $cert_id;
    $postfields["CSR"] = $csr;
    $postfields["command"] = "CertParseCSR";
    $postfields["ResponseType"] = "XML";
    $result = enomssl_call($postfields, $testmode);
    $values["error"] = $result["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
    if ($values["error"]) return ["status"=>"error","message"=>$values["error"]];

    $values["displaydata"]["Organization"] = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["ORGANIZATION"];
    $values["displaydata"]["Organization Unit"] = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["ORGANIZATIONUNIT"];
    $values["displaydata"]["Email"] = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["EMAIL"];
    $values["displaydata"]["Locality"] = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["LOCALITY"];
    $values["displaydata"]["State"] = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["STATE"];
    $values["displaydata"]["Country"] = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["COUNTRY"];
    return $values;
}
function enomssl_SSLStepThree($params)
{
    $product = $params["product"];
    $client = $params["client"];
    $configurations = $product["configurations"];    
    $testmode = (@$configurations["configoption5"] == "yes") ? true:false;

    $orderid = $params["remoteid"];
    $cert_id = $_SESSION["enomsslcert"][$orderid]["id"];
    $webservertype = $params["servertype"];
    $csr = $params["csr"];
    $firstname = $params["firstname"];
    $lastname = $params["lastname"];
    $organisationname = $params["organisationname"];
    $jobtitle = $params["jobtitle"];
    $emailaddress = $params["email"];
    $address1 = $params["address1"];
    $address2 = $params["address2"];
    $city = $params["city"];
    $state = $params["state"];
    $postcode = $params["postcode"];
    $country = $params["country"];
    $phonenumber = $params["phonenumber"];
    $faxnumber = $params["faxnumber"];
    $uccDomains = isset($params["fields"]["ucc_domains"]) ? $params["fields"]["ucc_domains"] : "";
    $uccEmails = isset($params["fields"]["ucc_emails"]) ? $params["fields"]["ucc_emails"] : "";
    $approveremail = $params["approveremail"];
    $cert_id = $_SESSION["enomsslcert"][$orderid]["id"];
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    $postfields["CertID"] = $cert_id;
    $postfields["CSR"] = $csr;
    $postfields["command"] = "CertParseCSR";
    $postfields["ResponseType"] = "XML";
    $result = enomssl_call($postfields, $testmode);
    $csr_organization = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["ORGANIZATION"];
    $csr_organizationunit = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["ORGANIZATIONUNIT"];
    $csr_email = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["EMAIL"];
    $csr_locality = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["LOCALITY"];
    $csr_state = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["STATE"];
    $csr_country = $result["INTERFACE-RESPONSE"]["CERTPARSECSR"]["COUNTRY"];
    $postfields = array();
    $postfields["uid"] = $configurations["configoption1"];
    $postfields["pw"] = $configurations["configoption2"];
    if ($params["approveremail"]) {
        $postfields["ApproverEmail"] = $params["approveremail"];
    }
    $postfields["CertID"] = $cert_id;
    $postfields["CSRAddress1"] = $address1;
    $postfields["CSRPostalCode"] = $postcode;
    if ($csr_organization) {
        $postfields["CSROrganization"] = $csr_organization;
    }
    if ($csr_organizationunit) {
        $postfields["CSROrganizationUnit"] = $csr_organizationunit;
    }
    if ($csr_locality) {
        $postfields["CSRLocality"] = $csr_locality;
    }
    if ($csr_state) {
        $postfields["CSRStateProvince"] = $csr_state;
    }
    if ($csr_country) {
        $postfields["CSRCountry"] = $csr_country;
    }
    if ($uccDomains) {
        $count = 0;
        $uccEmails = explode("\r\n", $uccEmails);
        foreach (explode("\r\n", $uccDomains) as $key => $domain) {
            $count++;
            $postfields["UCCDomainList" . $count] = $domain;
            $postfields["UCCEmailList" . $count] = $uccEmails[$key];
        }
        $postfields["DomainListNumber"] = $count;
    }
    $postfields["command"] = "CertPurchaseCert";
    $postfields["ResponseType"] = "XML";
    $result = enomssl_call($postfields, $testmode);
    $values["error"] = $result["INTERFACE-RESPONSE"]["ERRORS"]["ERR1"];
    if ($values["error"]) return ["status"=>"error","message"=>$values["error"]];

    unset($_SESSION["enomsslcert"]);
    return $values;
}
function enomssl_call($fields, $testmode = "")
{
    $url = $testmode ? "resellertest.enom.com" : "reseller.enom.com";
    $query_string = "";
    $whmcsVersion = App::getVersion();
    $fields["Engine"] = "WHMSOLS" . GetWhmsolsVersion();
    foreach ($fields as $k => $v) {
        $query_string .= (string) $k . "=" . urlencode($v) . "&";
    }
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, "https://" . $url . "/interface.asp");
    curl_setopt($ch, CURLOPT_POSTFIELDS, $query_string);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_TIMEOUT, 100);
    $data = curl_exec($ch);
    if (curl_error($ch)) {
        return ["status"=>"error","message"=>"CURL Error: ".curl_errno($ch)." - ".curl_error($ch)];
    }
    curl_close($ch);
    $result = XMLtoARRAY($data);
    logModuleCall("enomssl", $fields["command"], $fields, $result, "", array($fields["uid"], $fields["pw"]));
    return $result;
}